package com.learning.examples.TimerJCGExample;

import java.util.Calendar;
import java.util.Timer;
import java.util.TimerTask;

public class TimerExample {
	private String healthStatus = "GREEN";

	public static void main(String[] args) {

		TimerExample example = new TimerExample();
		example.setHealthStatus("GREEN");

		// Create the Timer object
		Timer timer = new Timer("JCG Timer Example");

		// Create Timer task created to send heartBeats
		TimerTask taskToExecute = new TimerTaskSendHeartBeat(example);
		// schedule the task to start executing after 1 second
		// and re-execute every 10 seconds
		timer.scheduleAtFixedRate(taskToExecute, 1000, 10000);

		// Create Timer task to setHeartBeatStatus
		TimerTask setHeartBeatStatus = new TimerTaskUpdateHeartBeat(example);
		// schedule the task to start immediately but execute
		// first time after 30 seconds
		timer.schedule(setHeartBeatStatus, 30000);

		// Wait for 60 seconds and then cancel the timer cleanly
		try {
			Thread.sleep(60000);
		} catch (InterruptedException e) {
		}
		System.out.println("Cancelling Timer Cleanly after 60 seconds");
		timer.cancel();
	}

	/**
	 * Get Heartbeat Status of the application, could be GREEN / AMBER / RED
	 * based on any exceptions or service health
	 * 
	 * @return String
	 */
	public String getHealthStatus() {
		return this.healthStatus;
	}

	/**
	 * Set the status for the application could be GREEN / AMBER / RED
	 * 
	 * @param healthStatus
	 */
	public void setHealthStatus(String healthStatus) {
		this.healthStatus = healthStatus;
	}
}

class TimerTaskUpdateHeartBeat extends TimerTask {

	TimerExample healthClass = null;

	public TimerTaskUpdateHeartBeat(TimerExample healthClass) {
		this.healthClass = healthClass;
	}

	@Override
	public void run() {
		System.out.println("Task 2:: 30 seconds completed :: Updating health "
				+ "status to AMBER");
		healthClass.setHealthStatus("AMBER");
	}

}

class TimerTaskSendHeartBeat extends TimerTask {

	TimerExample healthStatusHolder = null;

	public TimerTaskSendHeartBeat(TimerExample healthStatusHolder) {
		this.healthStatusHolder = healthStatusHolder;
	}

	HeartBeatMessage message = null;

	@Override
	public void run() {

		// create HeartBeat message by getting Health Status (RED/GREEN/AMBER)
		// Error Code, if any AND time at which heartbeat is sent to help
		// receiver discard any delayed messages due to latency
		message = new HeartBeatMessage(
				this.healthStatusHolder.getHealthStatus(), Calendar
						.getInstance().getTimeInMillis(), -1);

		System.out.println("Sending HeartBeat Message");

		// Send the message to Monitoring Dashboard
		System.out.println(message);

		System.out.println("HeartBeat Message Sent");
	}

	/**
	 * Simple POJO which is a heartbeat message object It can have any decoder
	 * encoder mechanism to send over any messaging platform
	 */
	class HeartBeatMessage {
		private String status;
		private long heartBeatTime;
		private int errorCode;

		public HeartBeatMessage(String status, long heartBeatTime, int errorCode) {
			this.status = status;
			this.heartBeatTime = heartBeatTime;
			this.errorCode = errorCode;
		}

		public String getStatus() {
			return status;
		}

		public long getHeartBeatTime() {
			return heartBeatTime;
		}

		public int getErrorCode() {
			return errorCode;
		}

		@Override
		public String toString() {
			return "status: " + status + " timeOfHeartBeat: "
					+ new java.util.Date(this.heartBeatTime) + " errCode : "
					+ this.errorCode;
		}
	}
}
